#!/usr/bin/python3

import sys
import os
import re
from xtable import xtable
import xmltodict
import argparse
import json
import traceback

mysqlcmds = dict()
mysqlcmds["show databases"] = "select * from information_schema.schemata"
mysqlcmds["show tables"] = "select table_schema, table_name, table_type type, engine, table_rows, avg_row_length, data_length, coalesce(update_time,create_time) last_change, check_time last_chk from information_schema.tables where database() is NULL or table_schema = database()"



def mysqlx_main():
    parser = argparse.ArgumentParser(description="yet another mysql client CLI tool/shell")
    parser.add_argument(
        "-v",
        "--pivot",
        dest="pivot",
        action="store_true",
        default=False,
        help="pivot wide tables.",
    )
    parser.add_argument(
        "-w",
        "--widthhint",
        dest="widthhint",
        default=None,
        help="hint for col width. '0:20,2:30,'",
    )
    parser.add_argument(
        "-p",
        "--page",
        dest="page",
        type=int,
        default=2**30,
        help="rows per page. print header line again",
    )
    parser.add_argument(
        "-X",
        "--debug",
        dest="debug",
        action="store_true",
        default=False,
        help="debug mode",
    )
    parser.add_argument(
        "--csv",
        dest="csv",
        action="store_true",
        default=False,
        help="dump as CSV",
    )
    parser.add_argument(
        "--json",
        dest="json",
        action="store_true",
        default=False,
        help="dump as json",
    )
    parser.add_argument(
        "--yaml",
        dest="yaml",
        action="store_true",
        default=False,
        help="dump as yaml",
    )
    parser.add_argument(
        "--markdown",
        dest="markdown",
        action="store_true",
        default=False,
        help="dump as markdown",
    )
    parser.add_argument(
        "--html",
        dest="html",
        action="store_true",
        default=False,
        help="dump as html",
    )
    parser.add_argument(
        "--plain",
        "--nocolor",
        dest="plain",
        action="store_true",
        default=False,
        help="no ansi color",
    )
    parser.add_argument(
        "--nowrap",
        "--nowrap",
        dest="nowrap",
        action="store_true",
        default=False,
        help="when specified, output will be limited to current terminal width",
    )
    parser.add_argument(
        "--wrap",
        "--wrap",
        dest="wrap",
        action="store_true",
        default=False,
        help="wrap mode. widthhint will be disabled in this mode.",
    )
    parser.add_argument(
        "--timeout",
        dest="timeout",
        type=int,
        default=1,
        help="read timeout. default 1s",
    )
    args = parser.parse_args()

    if args.plain:
        os.environ["force_ansicolor"] = "0"

    def xtimeout_call(fn=None, msg="timeouted.",exitfn=None) :
        import signal
        TIMEOUT = 1
        def interrupted(signal, frame):
            print("# {}".format(msg), file=sys.stderr, flush=True)
            if exitfn :
                exitfn()
            sys.exit(-1)
        signal.signal(signal.SIGALRM, interrupted)
        signal.alarm(TIMEOUT)
        res = None
        if fn :
            res = fn()
        signal.alarm(0)
        return res
    xinput = xtimeout_call(sys.stdin.read, "Timeout getting input from STDIN.", parser.print_help)
    xinput = xinput.strip()
    if not (xinput.lower().startswith("select") \
      or xinput.lower().startswith("update") \
      or xinput.lower().startswith("insert") \
      or xinput.lower().startswith("delete") \
      or xinput.lower().startswith("with") ) :
          k = re.sub(r"\s+"," ",xinput.lower())
          if k in mysqlcmds :
              xinput = mysqlcmds[k]
          else :
              xinput = None
    print(xinput)



if __name__ == "__main__":
    mysqlx_main()



from . import xml_formatter_main, mysqlx_main
from importlib_metadata import version

if __name__ == "__main__" :
    print("# mysql-toolbox, version {}".format(version('mysql-toolbox')))
    print("# by Yonghang Wang, 2021")
    print("mysql-xml-formatter       -  formatting 'mysql --xml' output.")
    print("mysql-replication-display -  replicaiton topology.")



#!/usr/bin/python3

import sys
import os
import re
import argparse
import json
import traceback
import configparser
import pymysql
import yaml
from collections import defaultdict

def repl_main():
    parser = argparse.ArgumentParser(description="yet another mysql client CLI tool/shell")
    parser.add_argument( "-s", "--source", dest="source", default=None, help="caculate replication topo from",)
    parser.add_argument( "-P", "--pool", dest="pool", default=None, help="full server list.",)
    parser.add_argument( "--domain", dest="domain", default=None, help="when matching, removing the domain part",)
    parser.add_argument( "--detail", dest="detail", action="store_true", default=False, help="show detail",)
    parser.add_argument( "-X", "--debug", dest="debug", action="store_true", default=False, help="debug mode",)
    args = parser.parse_args()

    config = configparser.ConfigParser()
    config.read(os.path.expanduser("~/.my.cnf"))
    config = config._sections

    dftuser   = config.get('client',dict()).get('user',None)
    dftpasswd = config.get('client',dict()).get('password',None)
    dfthost   = config.get('client',dict()).get('host',None)
    dftport   = config.get('client',dict()).get('port',None)
    dftdb     = config.get('client',dict()).get('database',None)

    dbpool = dict()

    def get_repl(host) :
        user   = config.get(host,dict()).get('user',None) if args.source in config  else dftuser
        passwd = config.get(host,dict()).get('password',None) if args.source in config else dftpasswd
        port   = (config.get(host,dict()).get('port',3306) if args.source in config else dftport) or 3306
        db     = (config.get(host,dict()).get('db',None) if args.source in config else dftdb) or "mysql"
        if args.debug :
            conn_paras = {"user":user,"password":passwd,"host":host,"port":port,"database":db}
            print("# conn_paras : ", json.dumps(conn_paras))
        repl = dict()
        repl["HOST"] = host
        dbconn = pymysql.connect(host=host,user=user,password=passwd,database=db)
        with dbconn :
            with dbconn.cursor() as cursor :
                id_sql = "select @@server_id as svrid, @@server_uuid as svruuid"
                cursor.execute(id_sql)
                row = cursor.fetchone()
                repl["server_id"] = row[0]
                repl["server_uuid"] = row[1]
                #repl["comboid"] = "{}/{}/{}".format(host,row[0],row[1])
                repl["comboid"] = "{}/{}".format(host,row[0])
                downstream_sql = "select * from performance_schema.threads where 1=1 and processlist_command = 'Binlog Dump GTID'"
                cursor.execute(downstream_sql)
                columns = [ col[0] for col in cursor.description]
                rows = cursor.fetchall()
                for r in rows :
                    if "downstreams" not in repl :
                        repl['downstreams'] = list()
                    repl['downstreams'].append(dict(zip(columns,r)))
                upstream_sql = "select c.*, s.thread_id, s.service_state from performance_schema.replication_connection_configuration c, performance_schema.replication_connection_status s where 1=1"
                cursor.execute(upstream_sql)
                columns = [ col[0] for col in cursor.description]
                rows = cursor.fetchall()
                for r in rows :
                    if "upstreams" not in repl :
                        repl['upstreams'] = list()
                    repl['upstreams'].append(dict(zip(columns,r)))
        return repl

    def collect_repl(srchost) :
        if args.domain :
            srchost = srchost.replace("."+args.domain,"")
            srchost = srchost.replace(args.domain,"")
        if not srchost or srchost in dbpool :
            return
        src = get_repl(srchost)
        dbpool[srchost] = src
        for r in src.get("downstreams",[])  :
            rh = r["PROCESSLIST_HOST"]
            if args.domain :
                 rh = rh.replace("."+args.domain,"")
                 rh = rh.replace(args.domain,"")
            collect_repl(rh)
        for r in src.get("upstreams",[])  :
            rh = r["HOST"]
            if args.domain :
                 rh = rh.replace("."+args.domain,"")
                 rh = rh.replace(args.domain,"")
            collect_repl(rh)

    if args.source :
        #src = get_repl(args.source)
        collect_repl(args.source)
        
    if args.pool:
        for host in [src for src in args.pool.split(",") if src ] :
            if host in dbpool :
                continue
            collect_repl(host)

    if args.debug :
        print(json.dumps(dbpool,indent=2))

    heads = list()
    if args.source :
        heads.append(args.source)
    else :
        heads += [ host for host, repl in dbpool.items() if "upstreams" not in repl ]
    if args.debug :
        print("# heads = ", json.dumps(heads))

    details = list()

    visited=set()
    def tree_print(head, shft="", running=True) :
        if head is None :
            return
        if shft :
            if running :
                #print(shft+"---> "+head)
                print(shft+"---> "+dbpool[head]["comboid"])
            else :
                #print(shft+"xx-> "+head)
                print(shft+"-x-> "+dbpool[head]["comboid"])
        else :
            #print(head)
            print(dbpool[head]["comboid"])
        details.append((head,shft))
        if head in visited :
            return
        visited.add(head)
        for h,repl in dbpool.items() :
            if "upstreams" not in repl :
                continue
            ischild = False
            running = False
            for r in repl["upstreams"] :
                if r["HOST"] != head :
                    continue
                ischild = True
                running = r['service_state'] == "ON"
                break
            if not ischild :
                continue
            tree_print(h,shft+"    ",running)

    for h in heads :
        tree_print(h)
                
    if args.detail :
        for h, shft in details :
            print(yaml.safe_dump(dbpool[h]))

if __name__ == "__main__" :
    repl_main()








#!/usr/bin/python3

import sys
import os
import re
from xtable import xtable
import xmltodict
import argparse
import json
import traceback


def xml_formatter_main():
    parser = argparse.ArgumentParser(description="format 'mysql --xml' output")
    parser.add_argument(
        "-I",
        "--includes",
        dest="includes",
        default=list(),
        action="append",
        help="when defined only include column names match RE.",
    )
    parser.add_argument(
        "-E",
        "--excludes",
        dest="excludes",
        default=list(),
        action="append",
        help="when defined exclude column names match RE.",
    )
    parser.add_argument(
        "-R",
        "--preprocessing",
        "--replacements",
        dest="preprocessing",
        default=list(),
        action="append",
        help=
        "to mask/modify field values before processing. useful when need masking or some special characters in column such as authentication_string may break the following processing. in format of 'field=xxx,field2=xxx2'",
    )
    parser.add_argument(
        "-v",
        "--pivot",
        dest="pivot",
        action="store_true",
        default=False,
        help="pivot wide tables.",
    )
    parser.add_argument(
        "-w",
        "--widthhint",
        dest="widthhint",
        default=None,
        help="hint for col width. '0:20,2:30,'",
    )
    parser.add_argument(
        "-p",
        "--page",
        dest="page",
        type=int,
        default=2**30,
        help="rows per page. print header line again",
    )
    parser.add_argument(
        "-X",
        "--debug",
        dest="debug",
        action="store_true",
        default=False,
        help="debug mode",
    )
    parser.add_argument(
        "--csv",
        dest="csv",
        action="store_true",
        default=False,
        help="dump as CSV",
    )
    parser.add_argument(
        "--json",
        dest="json",
        action="store_true",
        default=False,
        help="dump as json",
    )
    parser.add_argument(
        "--yaml",
        dest="yaml",
        action="store_true",
        default=False,
        help="dump as yaml",
    )
    parser.add_argument(
        "--markdown",
        dest="markdown",
        action="store_true",
        default=False,
        help="dump as markdown",
    )
    parser.add_argument(
        "--html",
        dest="html",
        action="store_true",
        default=False,
        help="dump as html",
    )
    parser.add_argument(
        "--plain",
        "--nocolor",
        dest="plain",
        action="store_true",
        default=False,
        help="no ansi color",
    )
    parser.add_argument(
        "--cutwrap",
        "--cutwrap",
        dest="cutwrap",
        action="store_true",
        default=False,
        help="when specified, output will be limited to current terminal width",
    )
    parser.add_argument(
        "--wrap",
        "--wrap",
        dest="wrap",
        action="store_true",
        default=False,
        help="wrap mode. widthhint will be disabled in this mode.",
    )
    parser.add_argument(
        "--timeout",
        dest="timeout",
        type=int,
        default=1,
        help="read timeout. default 1s",
    )
    args = parser.parse_args()

    if args.plain:
        os.environ["force_ansicolor"] = "0"

    import signal
    TIMEOUT = 1

    def interrupted(signal, frame):
        print("# timeout/no input from STDIN detected.",
              file=sys.stderr,
              flush=True)
        parser.print_help()
        sys.exit(0)

    signal.signal(signal.SIGALRM, interrupted)
    signal.alarm(TIMEOUT)
    xmlinput = sys.stdin.read()
    signal.alarm(0)

    if len(args.preprocessing) > 0:
        pairs = dict()
        for pre in args.preprocessing:
            for p in [p for p in pre.split(",") if p and '=' in p]:
                field, value = p.split('=', 1)
                if field:
                    pairs[field] = value or ""
        newxml = xmlinput
        for field, value in pairs.items():
            newxml = re.sub(
                r"<field name=\"{}\">.*?</field>".format(field),
                "<field name=\"{}\">{}</field>".format(field, value), newxml,
                re.DOTALL)
        xmlinput = newxml

    try:
        doc = xmltodict.parse(xmlinput)
    except:
        xmldata = [str(lno) + " : " + ln for lno, ln in enumerate(xmlinput.splitlines())]
        errmsg = traceback.format_exc().splitlines()[-1]
        m = re.search(r"not well-formed \(invalid token\): line (\d+),",errmsg) 
        if m :
            lno = int(m.group(1))
            begin = max(lno-3,0)
            end = lno+4
            print("\n".join(xmldata[begin:end]),file=sys.stderr,flush=True)
        else :
            print("{}".format("\n".join(xmldata)), file=sys.stderr, flush=True)
        print("# error parsing XML contents : {}".format(errmsg), file=sys.stderr, flush=True)
        return -1
    if "resultset" not in doc:
        print("# not valid resultset XML contents :",
              file=sys.stderr,
              flush=True)
        print(xmlinput, file=sys.stderr, flush=True)
        return -1

    rs = doc["resultset"]
    if args.debug:
        print("# resultset section : \n",
              json.dumps(rs, indent=2),
              file=sys.stderr,
              flush=True)
    if "row" not in rs or len(rs["row"]) == 0:
        print("# found no data :", file=sys.stderr, flush=True)
        return -1

    rows = rs["row"]
    if type(rows) is not list:
        rows = [rows]
    tbl = list()
    for r in rows:
        if args.debug:
            print("# row :", r, file=sys.stderr, flush=True)
        t = {}
        e = r.get("field", [])
        if type(e) is not list:
            e = [e]
        for c in e:
            if len(args.includes) > 0:
                found = False
                for sect in args.includes:
                    for inx in [s for s in sect.split(",") if s]:
                        if re.search(r"{}".format(inx), c.get('@name'),
                                     re.IGNORECASE):
                            found = True
                            break
                if not found:
                    continue
            if len(args.excludes) > 0:
                found = False
                for sect in args.excludes:
                    for ex in [s for s in sect.split(",") if s]:
                        if re.search(r"{}".format(ex), c.get('@name'),
                                     re.IGNORECASE):
                            found = True
                            break
                if found:
                    continue
            t[c.get('@name')] = c.get('#text')
        tbl.append(t)

    if args.debug:
        print("# table data prepared :\n",
              json.dumps(tbl, indent=2),
              file=sys.stderr,
              flush=True)

    if len(tbl) == 0:
        print("# empty result set.", file=sys.stderr, flush=True)
        return 0

    xt = xtable.init_from_list(tbl)
    if args.widthhint or args.page != 2**30 or args.wrap or args.cutwrap :
        xt = xtable(data=xt.get_data(),
                    header=xt.get_header(),
                    widthhint=args.widthhint,
                    superwrap=args.wrap,
                    cutwrap=args.cutwrap,
                    debug=args.debug,
                    rowperpage=args.page)
    if args.pivot:
        print(xt.pivot())
    elif args.csv:
        print(xt.csv())
    elif args.json:
        print(xt.json())
    elif args.yaml:
        print(xt.yaml())
    elif args.markdown:
        print(xt.markdown())
    elif args.html:
        print(xt.html())
    else:
        print(xt)


if __name__ == "__main__":
    xml_formatter_main()
